package com.vazil.vridge.docs.service;

import com.vazil.vridge.docs.model.GuideTest;
import com.vazil.vridge.docs.repository.GuideTestRepository;
import com.vazil.vridge.docs.utils.TimeManager;
import lombok.AllArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
@AllArgsConstructor
@Log4j2
public class GuideTestService {
    GuideTestRepository repository;

    public List<GuideTest> findGuideIndex() throws Exception{
        List<GuideTest> guideList = repository.findAllByOrderByOrder();
        return guideList;
    }

    @Transactional
    public GuideTest findGuideById(String guideId) throws Exception{
        GuideTest targetGuide = repository.findById(guideId).orElseThrow(NoSuchElementException::new);

        int viewCount = targetGuide.getView() != null ? targetGuide.getView() : 0;
        targetGuide.setView(viewCount + 1);
        repository.save(targetGuide);

        targetGuide.setNextGuide(getNextGuide(targetGuide, false));
        targetGuide.setPrevGuide(getPrevGuide(targetGuide));

        return targetGuide;
    }

    public GuideTest getPrevGuide(GuideTest targetGuide) throws Exception{
        String path = targetGuide.getPath();
        String parentPath = path.substring(0, path.lastIndexOf("/", path.length() -2) + 1);

        // depth == 1, order == 0 인 경우 이전 guide 없음
        if(targetGuide.getDepth() == 1 && targetGuide.getOrder() == 0)
            return null;

        // order == 0 인 경우 부모 guide 반환
        if(targetGuide.getOrder() == 0){
            return repository.findByPath(parentPath);
        }

        // 이전 형제의 자식이 없을 경우 해당 형제 반환
        GuideTest prevSibling = repository.findByPathContainingAndDepthAndOrder(parentPath, targetGuide.getDepth(), targetGuide.getOrder() - 1);
        if(repository.findAllByPathContainingAndDepth(prevSibling.getPath(), prevSibling.getDepth() + 1).size() == 0){
            return prevSibling;
        } else { // 이전 형제의 자식이 있을 경우 가장 마지막 자식 반환
            return getLastChildGuide(prevSibling);
        }
    }

    public GuideTest getLastChildGuide(GuideTest targetGuide) throws Exception{
        GuideTest lastChild = repository.findAllByPathContainingAndDepthOrderByOrderDesc(targetGuide.getPath(), targetGuide.getDepth() + 1).get(0);

        if(repository.findAllByPathContainingAndDepth(lastChild.getPath(), lastChild.getDepth() + 1).size() == 0){
            return lastChild;
        } else {
            return getLastChildGuide(lastChild);
        }
    }


    /**
     * @param targetGuide
     * @param onlySibling true일 경우, 자식 guide는 무시하고 다음 형제 guide를 찾음
     */
    public GuideTest getNextGuide(GuideTest targetGuide, Boolean onlySibling) throws Exception{
        String path = targetGuide.getPath();
        String parentPath = targetGuide.getDepth() == 1 ?
                "/" : path.substring(0, path.lastIndexOf("/", path.length() -2) + 1);
        GuideTest nextGuide = null;

        // child가 있으면 order == 0 인 자식 반환 (onlySibling == true 일 경우 패스하여 형제 반환으로 넘어감)
        if(!onlySibling){
            nextGuide = repository.findByPathContainingAndDepthAndOrder(path, targetGuide.getDepth() + 1, 0);
        }

        // child가 없으면 다음 형제 반환
        if(nextGuide == null){
            nextGuide = repository.findByPathContainingAndDepthAndOrder(parentPath, targetGuide.getDepth(), targetGuide.getOrder() + 1);
        }

        // child가 없고, 다음 형제 없는 경우, 부모 guide의 nextGuide 탐색
        if(nextGuide == null && targetGuide.getDepth() != 1) {
            GuideTest parentGuide = repository.findByPath(parentPath);
            nextGuide = getNextGuide(parentGuide, true);
        }

        return nextGuide;
    }

    @Transactional
    public GuideTest createGuide(GuideTest guide) throws Exception{
//        guide.setOrder(guide.getOrder() == null ? getLastOrder(guide.getParentId()) : 0);
        guide.setLike(0);
        guide.setView(0);
        guide.setCreateDate(TimeManager.now());
        return repository.save(guide);
    }

    @Transactional
    public GuideTest updateGuide(GuideTest guide) throws Exception{
        GuideTest savedGuide = repository.findById(guide.getId()).orElseThrow(NoSuchElementException::new);

        savedGuide.setOrder(guide.getOrder());
        savedGuide.setUpdateDate(TimeManager.now());

        return repository.save(savedGuide);
    }
}
